@@ -4,11 +4,14 @@
 package ethtool
 
 import (
+	"encoding/binary"
 	"errors"
 	"fmt"
 	"os"
+	"slices"
 	"strings"
 
+	"github.com/josharian/native"
 	"github.com/mdlayher/genetlink"
 	"github.com/mdlayher/netlink"
 	"golang.org/x/sys/unix"
@@ -148,6 +151,51 @@ func (c *client) linkMode(flags netlink.HeaderFlags, ifi Interface) ([]*LinkMode
 	return parseLinkModes(msgs)
 }
 
+// UpdateLinkMode updates the given Interface with the non-nil link mode properties in
+// the LinkModeUpdate.
+func (c *client) UpdateLinkMode(ifi Interface, lmu *LinkModeUpdate) error {
+	if lmu.Advertise != nil && lmu.Advertise.Sign() < 0 {
+		return errors.New("ethtool: can't update link mode, Advertise is invalid")
+	}
+	_, err := c.get(
+		unix.ETHTOOL_A_LINKMODES_HEADER,
+		unix.ETHTOOL_MSG_LINKMODES_SET,
+		netlink.Acknowledge,
+		ifi,
+		lmu.encode,
+	)
+	return err
+}
+
+// encode packs LinkModeUpdate data into the appropriate netlink attributes for the
+// encoder.
+func (lmu *LinkModeUpdate) encode(ae *netlink.AttributeEncoder) {
+	if lmu.SpeedMegabits != nil {
+		ae.Uint32(unix.ETHTOOL_A_LINKMODES_SPEED, uint32(*lmu.SpeedMegabits))
+	}
+	if lmu.Duplex != nil {
+		ae.Uint8(unix.ETHTOOL_A_LINKMODES_DUPLEX, uint8(*lmu.Duplex))
+	}
+	if lmu.Autoneg != nil {
+		ae.Uint8(unix.ETHTOOL_A_LINKMODES_AUTONEG, uint8(*lmu.Autoneg))
+	}
+	if lmu.Advertise != nil {
+		ae.Nested(unix.ETHTOOL_A_LINKMODES_OURS, func(nae *netlink.AttributeEncoder) error {
+			nae.Flag(unix.ETHTOOL_A_BITSET_NOMASK, true)
+			bitlen := lmu.Advertise.BitLen()
+			nae.Uint32(unix.ETHTOOL_A_BITSET_SIZE, uint32(bitlen))
+			b := make([]byte, ((bitlen+31)/32)*4)
+			b = lmu.Advertise.FillBytes(b)
+			if binary.ByteOrder(native.Endian) == binary.LittleEndian {
+				// FillBytes is big endian, reverse bytes for host order
+				slices.Reverse(b)
+			}
+			nae.Bytes(unix.ETHTOOL_A_BITSET_VALUE, b)
+			return nil
+		})
+	}
+}
+
 // LinkStates fetches link state data for all ethtool-supported links.
 func (c *client) LinkStates() ([]*LinkState, error) {
 	return c.linkState(netlink.Dump, Interface{})
@@ -687,6 +735,8 @@ func parseLinkModes(msgs []genetlink.Message) ([]*LinkMode, error) {
 				lm.SpeedMegabits = int(ad.Uint32())
 			case unix.ETHTOOL_A_LINKMODES_DUPLEX:
 				lm.Duplex = Duplex(ad.Uint8())
+			case unix.ETHTOOL_A_LINKMODES_AUTONEG:
+				lm.Autoneg = Autoneg(ad.Uint8())
 			}
 		}
 
